import time
from calendar import monthrange

from netforce.model import Model, fields, get_model
from netforce.access import get_active_company


class ReportAccountHDCaseSummary(Model):
    _name="clinic.report.account.hd.case.summary"
    _string="HD Case Report Summary"
    _transient=True
    
    _fields={
        "date": fields.Date("Month", required=True),
        "date_from": fields.Date("From", required=True),
        "date_to": fields.Date("To", required=True),
        'branch_id': fields.Many2One("clinic.branch","Branch"),
        'department_id': fields.Many2One("clinic.department","Departments"),
        'ptype_id': fields.Many2One("clinic.patient.type","Patient Type"),
        'reimbursable': fields.Selection([['yes','Yes'],['no','No']],'Reimbursable'),
    }

    def default_get(self,field_names=None,context={},**kw):
        defaults=context.get("defaults",{})
        date=defaults.get('date',time.strftime("%Y-%m-%d"))
        year,month=time.strftime("%Y-%m").split("-")
        weekday, total_day=monthrange(int(year), int(month))
        date_from=defaults.get('date_from','%s-%s-01'%(year,month))
        date_to=defaults.get('date_to',"%s-%s-%s"%(year,month,total_day))
        branch_id=defaults.get('branch_id',None)
        print('defaults ', defaults)
        if branch_id:
            branch_id=int(branch_id)
        department_id=defaults.get('department_id',None)
        if department_id:
            department_id=int(department_id)
        res={
            'date': date,
            'date_from': date_from,
            'date_to': date_to,
            'branch_id': branch_id,
            'department_id': department_id,
        }
        return res
    
    def get_report_data(self,ids,context={}):
        company_id=get_active_company()
        company=get_model("company").browse(company_id)
        defaults=self.default_get(context=context)
        date_from=defaults.get("date_from")
        date_to=defaults.get("date_to")
        date=defaults.get("date")
        branch_id=defaults.get("branch_id")
        department_id=defaults.get("department_id")
        ptype_id=None
        reimbursable=''
        if ids:
            obj=self.browse(ids)[0]
            branch_id=obj.branch_id.id
            department_id=obj.department_id.id
            date=obj.date 
            date_from=obj.date_from
            date_to=obj.date_to
            ptype_id=obj.ptype_id.id
            reimbursable=obj.reimbursable
        dom=[
            ['date','>=', date_from],
            ['date','<=', date_to],
        ]
        if ptype_id:
            dom.append(['patient_type_id','=',ptype_id])
        if branch_id:
            dom.append(['branch_id','=',branch_id])
        if department_id:
            dom.append(['department_id','=',department_id])
        if reimbursable=='yes':
            dom.append(['state','in',['waiting_payment']])
        elif reimbursable=='no':
            dom.append(['state','in',['paid']])
        else:
            dom.append(['state','in',['completed','waiting_payment','paid']])
        print('dom ', dom)    
        lines=[]
        for hdcase in get_model("clinic.hd.case").search_browse(dom):
            items={}
            for line in hdcase.lines:
                amt=line.amount or 0
                categ=line.product_categ_id
                code=(categ.code or "").lower()
                if not code in items.keys():
                    items[code]=[]
                items[code].append({
                    'amount': amt,
                })
            pm_number=''
            for pm in hdcase.payments:
                pm_number+=pm.number or ""
            inv_number=""
            for inv in hdcase.invoices:
                inv_number+=inv.number or ""

            patient=hdcase.patient_id
            ptype=hdcase.patient_type_id
            cycle=hdcase.cycle_id
            cycle_item=hdcase.cycle_item_id
            vals={
                'hdcase_id': hdcase.id,
                'number': hdcase.number or "",
                'date':  hdcase.date,
                'hct': hdcase.hct or 0,
                'ptype': ptype.name or "",
                'ptype_color': ptype.color or "default",
                'dname': hdcase.doctor_id.name or "",
                'cycle': cycle.name or "",
                'pname': patient.name,
                'cseq': cycle.sequence or 0,
                'cycle_item_id': cycle_item.id,
                'pm_number': pm_number,
                'inv_number': inv_number,
            }
            for code, item in items.items():
                vals.update({
                    code: sum([t['amount'] for t in item])
                })
            lines.append(vals)
        company_name=company.name or ""
        if department_id:
            company_name+=' (%s)'%get_model('clinic.department').browse(department_id).name or ""
        elif branch_id:
            company_name+=' (%s)'%get_model('clinic.branch').browse(branch_id).name or ""
        total_fee=0
        total_srv=0
        total_epo=0
        slines=[]
        no=1
        ptypes={}
        for line in sorted(lines, key=lambda x: (x['date'],x['cseq'])):
            ptype=line['ptype'] or ""
            if ptype not in ptypes.keys():
                ptypes[ptype]={
                    'qty': 0,
                    'color': line['ptype_color'] or "default",
                }
            ptypes[ptype]['qty']+=1
            total_fee+=line.get('fee',0)
            total_srv+=line.get('srv',0)
            total_epo+=line.get('epo',0)
            line['no']=no
            slines.append(line)
            no+=1
        types=[]
        total_qty=0
        for name, vals in ptypes.items():
            qty=vals['qty'] or 0
            total_qty+=qty
            color=vals['color'] or "default"
            types.append({
                'name': name,
                'qty': qty,
                'color': color,
            })
        data={
            'company_name': company_name,
            'branch_id': branch_id,
            'department_id': department_id,
            'date': date,
            'date_from': date_from,
            'date_to': date_to,
            'lines': slines,
            'total_fee': total_fee,
            'total_srv': total_srv,
            'total_epo': total_epo,
            'total_qty': total_qty,
            'types': types,
            'reimbursable': reimbursable,
        }
        return data

    def onchange_date(self,context={}):
        data=context['data']
        date=data['date']
        year,month,day=date.split("-")
        weekday, total_day=monthrange(int(year), int(month))
        data['date_from']="%s-%s-01"%(year,month)
        data['date_to']="%s-%s-%s"%(year,month,total_day)
        return data

    def onchange_branch(self,context={}):
        data=context['data']
        data['department_id']=None
        return data

ReportAccountHDCaseSummary.register()
